##########################################################
##                                                      ##
## INSTALL PROCEDURE FOR CMF/PLONE TYPES                ##
##                                                      ##
##                                                      ##
##########################################################


# EDIT FROM HERE...
from Products.PloneLoginHistory import install_globals, LoginHistoryTool
from Products.PloneLoginHistory.global_symbols import *
# ...TO HERE

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent, View, AccessContentsInformation
from Products.PythonScripts.PythonScript import PythonScript

from AccessControl.Permission import pname
import Globals

from cStringIO import StringIO
import string
import os

skin_name = "PloneLoginHistory"
perms_list = (LHT_view_all_history_Permission,LHT_view_all_lastlogin_Permission,LHT_view_my_history_Permission,LHT_view_my_lastlogin_Permission, )

configlets = \
( { 'id'         : 'ploneloginhistory'
  , 'name'       : 'Login History'
  , 'action'     : 'string:${portal_url}/prefs_login_history_view'
  , 'category'   : 'Products'
  , 'appId'      : 'PloneLoginHistory'
  , 'permission' : ManagePortal
  , 'imageUrl'  : 'logtool.gif'
  }
,
)

def addConfiglets(self, out):
    """Add Plone Login History to Control Panel 
    """
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
	for conf in configlets:
	    out.write('Adding configlet %s\n' % conf['id'])
	    configTool.registerConfiglet(**conf)

def removeConfiglets(self, out):
    """Remove Plone Login History from Control Panel 
    """
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
	for conf in configlets:
	    out.write('Removing configlet %s\n' % conf['id'])
	    configTool.unregisterConfiglet(conf['id'])

def install_tool(self,out):
    """ Install portal_login_history.
    """
    if not hasattr(self,'portal_login_history'):
	addTool = self.manage_addProduct['PloneLoginHistory'].manage_addTool
	addTool('Login History Tool')
	out.write('Plone Login History Tool successfully created\n')    
    else:
    	out.write('*** Plone Login History Tool already exists\n')    

def remove_tool(self,out):
    """ Uninstall portal_login_history.
    """
    if hasattr(self,'portal_login_history'):
	self._delObject('portal_login_history')
	out.write('Plone Login History Tool successfully removed\n')    
    else:
    	out.write('*** Plone Login History Tool not found\n')    


def installSubSkin(self, out, skinFolder):
    """ Install a subskin, i.e. a folder/directoryview.
    """
    skinsTool = getToolByName(self, 'portal_skins')

    for skin in skinsTool.getSkinSelections():
        path = skinsTool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinsTool.addSkinSelection( skin, path )
            out.write('Subskin successfully installed into %s.\n' % skin)    
        else:
            out.write('*** Subskin was already installed into %s.\n' % skin) 

def installPortalAction(self, out):
    """ Install Login History action.
    """
    actionTool = getToolByName(self, 'portal_actions')

    found=0
    for action in actionTool.listActions():
        if action.id=='complete_login_history':
	    found=1
            out.write('*** Portal Action Login History already exists')    
    
    if found==0:
	actionTool.addAction(
	            id = 'complete_login_history',
		    name = 'Login History',
		    action = 'string:${portal_url}/complete_login_history',
		    condition = None,
		    visible = 1,
		    permission = ('View login histories of all users'),
		    category = 'portal_tabs'
		    )
		    
	out.write('Portal Action LoginHistory installed.') 

def removePortalAction(self, out):
    """ remove Login History action.
    """
    actionTool = getToolByName(self, 'portal_actions')

    old_actions = actionTool._cloneActions()
    new_actions = []
    for action in old_actions:
	if action.id!='complete_login_history':
	    new_actions.append(action)
    actionTool._actions = tuple( new_actions )
    out.write('Portal Action LoginHistory removed.') 


def setupTypesandSkins(self, out):
    typesTool = getToolByName(self, 'portal_types')
    skinsTool = getToolByName(self, 'portal_skins')
    try:  
        addDirectoryViews(skinsTool, 'skins', install_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')

    installSubSkin(self, out, skin_name)


def setPermissions(self, out):
    """
    setPermissions(self, out) => Set standard permissions / roles
    """
    p = self

    # As a default behavior, newly-created permissions are granted to owner and manager.
    # To change this, just comment this code and grab back the code commented below to
    # make it suit your needs.
    for perm in perms_list:
        p.manage_permission(
            perm,
            ('Manager', 'Owner'),
            acquire = 1
            )

    out.write("Reseted default permissions\n")

def install_slot(self,out):
    """ Install Login Monitor.
    """
    done=0
    left_slots  = list(getattr(self,'left_slots'))
    right_slots = list(getattr(self,'right_slots'))
    myslot='here/login_monitor_slot/macros/loginMonitorBox'
    if myslot not in left_slots and myslot not in right_slots:
	done=1
	left_slots.append(myslot)
	self._updateProperty('left_slots',left_slots)
	out.write('Slot Login Monitor successfully installed\n')
    if done==0:
	out.write('Slot Login Monitor (here/login_monitor_slot/macros/loginMonitorBox) already exists\n')

def remove_slot(self,out):
    """ remove Login Monitor. We assume the Login Monitor slot ist not used twice
    """
    slots  = []
    slot_name =''
    myslot='here/login_monitor_slot/macros/loginMonitorBox'
    found=0
    
    left_slots  = list(getattr(self,'left_slots'))
    right_slots = list(getattr(self,'right_slots'))
    
    if myslot in left_slots:
	slots=left_slots
	slot_name='left_slots'

    if myslot in right_slots:
	slots=right_slots
	slot_name='right_slots'

    new=[]
    for slot in slots:
	if slot!=myslot:
	    new.append(slot)	
    if slot_name!='':
	self._updateProperty(slot_name,new)
	out.write('Slot Login Monitor successfully removed\n')
    else:
	out.write('*** There was no Login Monitor Slot to remove\n')
    
def uninstall(self):
    """Remove PloneLoginHistory tool
    """
    out=StringIO()
    remove_tool(self,out)
    removeConfiglets(self, out)
    removePortalAction(self, out)
    remove_slot(self, out)
    return out.getvalue()

def install(self):
    out=StringIO()
    addConfiglets(self, out)
    setPermissions(self, out)
    setupTypesandSkins(self, out)
    install_tool(self,out)
    installPortalAction(self, out)
    install_slot(self, out)
    out.write('Installation completed.\n')
    return out.getvalue()