from Globals import InitializeClass, DTMLFile
import string
import os
from Products.CMFCore.utils import UniqueObject, getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import ManagePortal
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.CMFPlone.PloneFolder import addPloneFolder
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo, getSecurityManager
from DateTime import DateTime
from global_symbols import *
from Products.CMFPlone.PloneUtilities import translate_wrapper
 
StorageDict = {}
StorageStrings = []

def HistoryStorages(*args):
    '''get supported History Storage modules'''
    global StorageStrings
    if StorageStrings == []:
	try:
	    list=os.listdir("%s/storages/" %(LHT_HOME))
	    try:
		list.remove('__init__.py')
	    except:
		pass
	    try:
	        list.remove('__init__.pyc')
	    except:
	        pass
	    try:
	        list.remove('__init__.pyo')
	    except:
	        pass
	    try:
	        list.remove('CVS')
	    except:
	        pass
	    for item in list:
	        if item not in StorageStrings:
		    StorageStrings.append(item)
	except:
	    StorageStrings = []
    return StorageStrings

HS=HistoryStorages()
for item in HS:
    try:
	Storage=__import__("Products.PloneLoginHistory.storages.%s.main" %item,globals(),globals(),("__doc__",))
	try:
	    obj = Storage.LoginHistoryStorage()
	    StorageDict[item] = obj
	except:
	    print 'import %s failed' %item
    except:
	import sys,traceback
	print '\nimport %s completely failed' %item
	t,v,tb=sys.exc_info()
	print t,v

def _getStorage(plugin_name):
    '''return Storage plugin'''
    ERROR=0
    Storage=None
    if StorageDict.has_key(plugin_name):
	try:
	    Storage = StorageDict[plugin_name]
	except:
	    ERROR = 'Error getting Storage plugin %s' %plugin_name
    else:
	ERROR='No Storage Plugin named %s' %plugin_name
    return Storage,ERROR

class LoginHistoryTool( UniqueObject, SimpleItem, PropertyManager ):
    """ Plone customized Login History Tool """
    id = 'portal_login_history'
    meta_type='Login History Tool'
    security = ClassSecurityInfo()
    
    _properties=(
	{'id':'DenyIfLogFails','type':'boolean','mode':'w'},
	{'id':'LogSuccessfulLogins','type':'boolean','mode':'w'},
	{'id':'LogFailedLogins','type':'boolean','mode':'w'},
	{'id':'EnableLog','type':'boolean','mode':'w'},
	{'id':'StorageToUse','type':'string','mode':'w'},
	{'id':'ListLimit','type':'int','mode':'w'},
    )
    
    manage_options = (
        PropertyManager.manage_options+SimpleItem.manage_options
    )
        
    def __init__(self):
	"""__init__(self,id,title='')"""
	self.DenyIfLogFails=0
	self.LogSuccessfulLogins=1
	self.LogFailedLogins=1
	self.EnableLog=1
	self.ListLimit=20
	self.StorageToUse='FileSystem'
	self.initStorageProperties()

    security.declareProtected(ManagePortal, "manage_makeChanges")
    def manage_makeChanges(self,EnableLog,DenyIfLogFails,LogSuccessfulLogins,LogFailedLogins,StorageToUse,ListLimit,REQUEST):
        """ change Properties """
	msg='Logging Options updated.'
	self.EnableLog=EnableLog
	self.ListLimit=int(ListLimit)
	self.DenyIfLogFails=DenyIfLogFails
	self.LogSuccessfulLogins=LogSuccessfulLogins
	self.LogFailedLogins=LogFailedLogins
	Storage,ERROR=_getStorage(StorageToUse)
	if ERROR==0:
	    self.StorageToUse=StorageToUse
	    storage_args=Storage.storage_arguments
	    for item in storage_args:
		k = item['name']
		val = REQUEST.get(k,item['default'])
		dt  = item['type']
		try:
		    self._setProperty(k,val,dt)
		except:
		    try:
			self._updateProperty(k,val)
		    except:
		        pass
	else:
	    msg = ERROR
	return msg

    security.declareProtected(ManagePortal, "getStoragePropertyValue")
    def getStoragePropertyValue(self,property_name):
        """ return storage property value """
	return getattr(self,property_name)

    security.declareProtected(ManagePortal, "getStorageArguments")
    def getStorageArguments(self):
        """ return arguments for storage plugin"""
	res=[]
	Storage=self._getStorage()
	if Storage is not None:
	    return Storage.storage_arguments
	return res

    security.declareProtected(ManagePortal, "getStorageNames")
    def getStorageNames(self):
        """ return list of all Storage Plugin names """
	return StorageStrings
	
    security.declareProtected(LHT_view_all_history_Permission, "getAllLoginHistories")
    def getAllLoginHistories(self,offset=0,REQUEST=None):
        """
        return all histories
        """
	Storage=self._getStorage()
	return Storage.getAllLoginHistories(self,offset)

    security.declareProtected(LHT_view_my_history_Permission, "getMyLoginHistory")
    def getMyLoginHistory(self,offset=0,REQUEST=None):
        """
        return login history for authenticated user
        """
	res=[]
	Storage=self._getStorage()
	username=''
	try:
	    username= getSecurityManager().getUser().getUserName()
	except:
	    pass
	if username!='':
	    return Storage.getMyLoginHistory(self,username,offset)
	return res

    security.declarePublic("getLastLogin")
    def getLastLogin(self,username,context,REQUEST=None):
        """
        return last login for given username
	"""
	res=None
	if _checkPermission(LHT_view_all_lastlogin_Permission,context)==1:
	    Storage=self._getStorage()
	    if username!='':
		return Storage.getLastLogin(self,username)
	return res

    security.declareProtected(LHT_view_all_lastlogin_Permission, "dummyViewAllLastLoginPermission")
    def dummyViewAllLastLoginPermission(self,REQUEST=None):
        """
        this is a dummy methode to introduce the permission 
	"""
	return None

    security.declareProtected(LHT_view_my_lastlogin_Permission, "getMyLastLogin")
    def getMyLastLogin(self,REQUEST=None):
        """
        return last login for authenticated user
	redundant implementation instead of just calling getLastLogin because of different permission requirements
	"""
	res=None
	Storage=self._getStorage()
	username=''
	try:
	    username= getSecurityManager().getUser().getUserName()
	except:
	    pass
	if username!='':
	    return Storage.getLastLogin(self,username)
	return res
	
    security.declareProtected(LHT_view_my_lastlogin_Permission, "getMyPreviousLogin")
    def getMyPreviousLogin(self,REQUEST=None):
        """
        return previous login for authenticated user ; if None user logged in first time
	"""
	res=None
	Storage=self._getStorage()
	username=''
	try:
	    username= getSecurityManager().getUser().getUserName()
	except:
	    pass
	if username!='':
	    return Storage.getMyPreviousLogin(self,username)
	return res


    security.declarePublic('log_login')            
    def log_login(self,user,success=0,REQUEST=None):
        """
        log login
	
        """
	Log(LOG_DEBUG,'success:%s user:%s' %(success,user))
	
	if self.EnableLog==0:
	    return
	if (self.LogSuccessfulLogins==0) and (success==1):
	    return
	if (self.LogFailedLogins==0) and (success==0):
	    return
	
	membership_tool = getToolByName(self,'portal_membership')
	member = membership_tool.getAuthenticatedMember() 

	login_date=DateTime()
        login_success='successful'
	login_user=member.getUserName()

	login_browser='unknown'
	login_ip='unknown'
	if success!=1:
	    login_success='FAILED'
	if REQUEST is not None:
	    if success!=1:
		login_user=user
	    try:
		login_browser=REQUEST['HTTP_USER_AGENT']
	    except:
		pass
	    try:
		login_ip=REQUEST['REMOTE_ADDR'] #if unpatched VirtualHostMonster used REMOTE_ADDR is always the local proxy
	        if login_ip=='127.0.0.1':
		    try:
			ip=REQUEST['HTTP_X_FORWARDED_FOR']
			if ip!='':
			    login_ip=ip
		    except:
			pass
	    except:
		pass
	self._storeLog(login_date,login_success,login_user,login_browser,login_ip)

    def _getStorage(self,REQUEST=None):
        """return storage and ERROR from StorageToUse"""
	Storage,ERROR=_getStorage(self.StorageToUse)
	return Storage

    def initStorageProperties(self):
        """init storage plugin properties"""
	Storage,ERROR = _getStorage(self.StorageToUse)
	if ERROR==0:
	    storage_args=Storage.storage_arguments
	    for item in storage_args:
		k = item['name']
		val = item['default']
		dt  = item['type']
		try:
		    self._setProperty(k,val,dt)
		except:
		    try:
			self._updateProperty(k,val)
		    except:
		        pass
	else:
	    return ERROR

    def _storeLog(self,login_date,login_success,login_user,login_browser,login_ip):
        """
        store to log
	
        """
	log_written=1
	try:
	    Storage=self._getStorage()
	    log_written=Storage.StoreLog(self,login_date,login_success,login_user,login_browser,login_ip)
	except:
	    log_written=0
	    
	if (self.DenyIfLogFails==1) and log_written==0:
	    raise 'Failed to write login history.'

    security.declarePublic('localized_time')            
    def localized_time(self,time = None, long_format = None, context = None):
        """
        translate DateTime ; return String or None
	
        """
	if time==None:
	    return None
	     		    
	comes_from='i18n'
	
	msgid = long_format and 'datetime_format_long' or 'datetime_format_short'
	# retrieve date format via translation service
	# format is C-style !
	format = translate_wrapper('PloneLoginHistory', msgid, context = context)
	    
	if format==None:
	    if context is not None:
	    # fallback to portal_properties if no msgstr received from
	    # translation service
		properties=context.portal_properties.site_properties
		if long_format:
		    format=properties.localLongTimeFormat
		    comes_from='properties.localLongTimeFormat'
		else:
		    format=properties.localTimeFormat
		    comes_from='properties.localTimeFormat'
	    else:
		comes_from='PloneLoginHistory hardcoded'
		if long_format:
		    format='%m-%d-%Y %H:%M:%S'
		else:
		    format='%m-%d-%Y'
	try:	
	    return DateTime(str(time)).strftime(format)
	except:
	    #raise 'Error localized_time','time: %s long_format:%s C-style format:%s comes from:%s' %(time,long_format,format,comes_from)
	    return str(time)
		
InitializeClass(LoginHistoryTool)